module Character exposing (..)

import Sphere exposing (Sphere)
import BoundedInt exposing (BoundedInt)
import Creation exposing (CreationPoints)

type alias Character =
    { name : String
    , creationPoints : CreationPoints
    , spheres : List Sphere
    , arete : BoundedInt
    }

new : Character
new =
    { name = "Default Name"
    , creationPoints = Creation.new
    , spheres = 
        [   { name = "Correspondence"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ,   { name = "Entropy"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ,   { name = "Forces"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ,   { name = "Life"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ,   { name = "Matter"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ,   { name = "Mind"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ,   { name = "Prime"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ,   { name = "Spirit"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ,   { name = "Time"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ]
    , arete = BoundedInt.new 1 0 10
    }

changeName : Character -> String -> Character
changeName character newName =
    if ( newName == "")
    then
        character
    else
        { character | name = newName }

changeSphere : Character -> Sphere -> Character
changeSphere character sphere =
    let
        maybeOldDots = Sphere.getDotsOfSphereInList character.spheres sphere.name
        difference = case maybeOldDots of
            Nothing -> 0
            Just oldDots ->
                if sphere.dots.value > character.arete.value
                then character.arete.value - oldDots.value
                else sphere.dots.value - oldDots.value
    in
        case maybeOldDots of
            Nothing -> character
            Just oldDots ->
                let
                    buyAmount = min difference character.creationPoints.spheres
                in
                    { character
                    | creationPoints = Creation.buySphere character.creationPoints buyAmount
                    , spheres = Sphere.changeSphereInList character.spheres ( Sphere.changeDotsOfSphereTo sphere ( oldDots.value + buyAmount ) )
                    }